package com.autonavi.alinkserver;

import android.content.Context;

import com.autonavi.alinkserver.config.AlinkConfig;
import com.autonavi.alinkserver.listener.AudioListener;
import com.autonavi.alinkserver.listener.EventListener;
import com.autonavi.alinkserver.listener.MicListener;
import com.autonavi.alinkserver.listener.StatusListener;
import com.autonavi.alinkserver.listener.TouchListener;
import com.autonavi.alinkserver.listener.VideoListener;

/**
 * ALinkServer SDK的入口类。
 *
 * @author Alink
 */
public class AlinkServer {
    /**
     * media音频类型常量。
     */
    public static final int AUDIO_TYPE_MEDIA = 1;

    /**
     * navigation音频类型常量。
     */
    public static final int AUDIO_TYPE_NAVIGATION = 2;

    /**
     * TTS音频类型常量。
     */
    public static final int AUDIO_TYPE_TTS = 3;

    private static final int INVALID_VALUE = -1;

    private AlinkConfig mALinkConfig = null;

    private AlinkServerWrapper mALinkServerWrapper = new AlinkServerWrapper();

    /**
     * SDK初始化。 设置SDK运行需要的参数。除了getVersion()方法之外，其他方法在此方法调用之后使用。
     *
     * @param context android进程的运行环境。
     * @param config  ALink必须的音视频参数。
     */
    public void onCreate(Context context, AlinkConfig config) {
        mALinkConfig = config;
        mALinkServerWrapper.init(context, config);
    }

    /**
     * 注册设备连接状态的监听类。
     *
     * @param listener 设备连接状态监听类。
     */
    public void registerStatusListener(StatusListener listener) {
        mALinkServerWrapper.registerStatusListener(listener);
    }

    /**
     * 获取设备的连接状态。
     *
     * @return 设备连接状态，连接状态的定义参见StatusListener接口。
     */
    public int getConnectionStatus() {
        int state = StatusListener.Status.STATUS_ALINK_NONE_CONNECTED;

        if (mALinkServerWrapper.isUsbConnected()) {
            state = StatusListener.Status.STATUS_ALINK_USB_CONNECTED;
        } else if (mALinkServerWrapper.isWifiConnected()) {
            state = StatusListener.Status.STATUS_ALINK_WIFI_CONNECTED;
        } else {
            // empty for design
        }

        return state;
    }

    /**
     * 注册Video的监听类。
     *
     * @param listener video的监听类。
     */
    public void registerVideoListener(VideoListener listener) {
        if (null == mALinkConfig || mALinkConfig.useTestVideo) {
            return;
        }

        mALinkServerWrapper.registerVideoListener(listener);
    }

    /**
     * 设备USB附件模式连接通知 设备USB以附件模式连接时，上层应用调用此接口通知SDK。SDK开始扫描USB连接，判断连接的是否为ALink车机端。
     * 如果是ALink车机端则开始与车机建立连接，如果不是，则放弃连接。
     */
    public void onUsbAccessoryAttached() {
        mALinkServerWrapper.scanUsbDevices();
    }

    /**
     * 注册Audio的监听类。 当前支持3种通道media，TTS，navigation，至少需要使用其中的一种。
     *
     * @param type     audio的类型。(AUDIO_TYPE_MEDIA，AUDIO_TYPE_TTS，AUDIO_TYPE_NAVIGATION)
     *                 分别录制media，TTS，navigation时，需分别注册不同的listener。
     * @param listener Audio的监听类。
     */
    public void registerAudioListener(int type, AudioListener listener) {
        if (null == mALinkConfig || mALinkConfig.useTestAudio) {
            return;
        }

        int rtpType = getRtpType(type);
        if (INVALID_VALUE == rtpType) {
            return;
        }

        mALinkServerWrapper.registerAudioListener(rtpType, listener);
    }

    /**
     * 注册TouchEvent的监听类。
     *
     * @param listener TouchEvent的监听类
     */
    public void registerTouchListener(TouchListener listener) {
        mALinkServerWrapper.registerTouchListener(listener);
    }

    /**
     * 注册Event的监听类。
     *
     * @param listener Event的监听类
     */
    public void registerEventListener(EventListener listener) {
        mALinkServerWrapper.registerEventListener(listener);
    }

    /**
     * 注册Mic的监听类。
     *
     * @param listener Mic的监听类
     */
    public void registerMicListener(MicListener listener) {
        mALinkServerWrapper.registerMicListener(listener);
    }

    /**
     * 发送事件给车机端。
     *
     * @param infomation json格式 {"event":1,"status":0} event=1表示asr事件，status=0开始，status=1停止
     */
    public void notifyInformation(String infomation) {
        mALinkServerWrapper.notifyInformation(infomation);
    }

    /**
     * 传输H264视频流。 只有VideoListener接收到onStart()回调后，该接口传输数据才有效，否则传输的内容会被SDK丢弃。
     *
     * @param length 视频流当前帧的长度。
     * @param buffer 视频流当前帧的内容buffer。
     */
    public void sendVideoData(int length, byte[] buffer) {
        if (null == mALinkConfig || mALinkConfig.useTestVideo) {
            return;
        }

        mALinkServerWrapper.onRecordVideo(buffer, length);
    }

    /**
     * 传输PCM音频流。 只有AudioListener接收到onStart()回调后，该接口传输才有效，否则传输的内容会被SDK丢弃。
     *
     * @param type   audio的类型，取值有AUDIO_TYPE_MEDIA，AUDIO_TYPE_TTS，AUDIO_TYPE_NAVIGATION三种
     * @param length 音频的长度。
     * @param buffer 音频的内容buffer。
     */
    public void sendAudioData(int type, int length, byte[] buffer) {
        if (null == mALinkConfig || mALinkConfig.useTestAudio) {
            return;
        }

        int rtpType = getRtpType(type);
        if (INVALID_VALUE == rtpType) {
            return;
        }

        mALinkServerWrapper.onRecordPcm(rtpType, buffer, length);
    }

    /**
     * 取消注册Audio的监听类。 取消注册之后type对应的声音流通道将不可用。
     *
     * @param type audio的类型，取值有AUDIO_TYPE_MEDIA，AUDIO_TYPE_TTS，AUDIO_TYPE_NAVIGATION三种。
     */
    public void unregisterAudioListener(int type) {
        if (null == mALinkConfig || mALinkConfig.useTestAudio) {
            return;
        }

        int rtpType = getRtpType(type);
        if (INVALID_VALUE == rtpType) {
            return;
        }

        mALinkServerWrapper.registerAudioListener(rtpType, null);
    }

    /**
     * 取消注册Video的监听类。 取消注册之后视频流通道不可用。
     */
    public void unregisterVideoListener() {
        if (null == mALinkConfig || mALinkConfig.useTestVideo) {
            return;
        }

        mALinkServerWrapper.registerVideoListener(null);
    }

    /**
     * 取消注册设备连接状态的监听。 取消注册之后SDK不再通知上层应用连接状态变更，不过调用getConnectionStatus()接口仍然有效。
     */
    public void unregisterStatusListener() {
        mALinkServerWrapper.registerStatusListener(null);
    }

    /**
     * 取消touch事件的监听。
     */
    public void unregisterTouchListener() {
        mALinkServerWrapper.registerTouchListener(null);
    }

    /**
     * 取消event事件的监听。
     */
    public void unregisterEventListener() {
        mALinkServerWrapper.registerEventListener(null);
    }

    /**
     * 取消mic的监听。
     */
    public void unregisterMicListener() {
        mALinkServerWrapper.registerMicListener(null);
    }

    /**
     * 销毁SDK资源。 调用onCreate方法后，结束SDK使用时必须调用此方法释放SDK资源。
     */
    public void onDestroy() {
        mALinkServerWrapper.destroy();
    }

    /**
     * 获取SDK版本号。
     *
     * @return SDK版本号。
     */
    public String getVersion() {
        return "1.0.0";
    }

    private int getRtpType(int audioType) {
        int rtpType = INVALID_VALUE;

        switch (audioType) {
            case AUDIO_TYPE_MEDIA:
                rtpType = AlinkConstant.RTP_TYPE_MUSIC;
                break;
            case AUDIO_TYPE_NAVIGATION:
                rtpType = AlinkConstant.RTP_TYPE_NAVIGATION;
                break;
            case AUDIO_TYPE_TTS:
                rtpType = AlinkConstant.RTP_TYPE_SPEECH;
                break;
            default:
                break;
        }

        return rtpType;
    }
}
